﻿$packageName = 'firebird'

$version = '3.0.2'
$url = 'https://downloads.sourceforge.net/project/firebird/firebird-win32/3.0.2-Release/Firebird-3.0.2.32703_0_Win32.exe'
$url64 = 'https://downloads.sourceforge.net/project/firebird/firebird-win64/3.0.2-Release/Firebird-3.0.2.32703_0_x64.exe'
$checksum = 'DC2250C8740ADFE1F0936A90102CC1546A189FFD2500984A90AD28EEDC4257CB'
$checksum64 = 'BBECFA0DA9DE5CFA52269A07EBDBC3A0C4D68BC028D0511CCEE3EB89FEECD07C'
$checksumType = 'sha256'
$installerName = 'Firebird-3.0.2.32703_0.exe'

$installerType = 'exe'
$installerFullName = Join-Path $env:TEMP $installerName

function Get-FirebirdPath
{
    $HKLMFirebirdInstancesKey = 'HKLM:\Software\Firebird Project\Firebird Server\Instances'
    $instances = Get-ItemProperty -Path "$HKLMFirebirdInstancesKey" -ErrorAction SilentlyContinue

    if (-not $instances) {
        # Not found. Try to search for a 32-bit install in a 64-bit architecture.

        $HKLMFirebirdInstancesKey = 'HKLM:\Software\Wow6432Node\Firebird Project\Firebird Server\Instances'
        $instances = Get-ItemProperty -Path "$HKLMFirebirdInstancesKey" -ErrorAction SilentlyContinue
    }

    if ($instances) {
        Write-Host "Firebird installation path: $($instances.DefaultInstance)"
        $instances.DefaultInstance
    } else {
        Write-Host "Firebird is NOT installed."
        $null
    }
}

function Download-FirebirdInstaller
{
    Write-Host "Downloading installer file: $installerFullName"
    Get-ChocolateyWebFile -PackageName $packageName -FileFullPath $installerFullName `
                          -Url $url -Url64bit $url64 `
                          -Checksum $checksum -ChecksumType $checksumType -Checksum64 $checksum64 -ChecksumType64 $checksumType
}

function Uninstall-Firebird
{
    $firebirdPath = Get-FirebirdPath

    if ($firebirdPath) {
        if (Get-Service -Name FirebirdServerDefaultInstance -ErrorAction SilentlyContinue) {
            Write-Host "Stopping Firebird service..."
            Stop-Service -Name FirebirdServerDefaultInstance
        }

        $uninstallers = Join-Path $firebirdPath 'unins*.exe'
        $lastUninstaller = Get-Item $uninstallers | Sort-Object LastWriteTime | Select-Object -Last 1
        $uninstallerArgs = '/VERYSILENT',
                           '/NORESTART',
                           '/SUPPRESSMSGBOXES'
        
        Write-Host "Calling uninstaller: $($lastUninstaller.FullName)"
        Uninstall-ChocolateyPackage -PackageName $packageName -FileType $installerType `
                                    -SilentArgs $uninstallerArgs -File $lastUninstaller.FullName
    }
}

function Install-Firebird
{
    Download-FirebirdInstaller

    Uninstall-Firebird

    if ($env:chocolateyPackageParameters -contains '/ClientOnly') {
        Write-Host 'Installing Firebird Client...'
        $serverTypeComponent = 'ClientComponent,'
        $serverTypeTask = ''
    } elseif ($env:chocolateyPackageParameters -contains '/ClientAndDevTools') {
        Write-Host 'Installing Firebird Client and Development tools...'
        $serverTypeComponent = 'ClientComponent,DevAdminComponent,'
        $serverTypeTask = ''
    } else {
        if ($version -lt '3') {
            # Firebird <= 2.5
            if ($env:chocolateyPackageParameters -contains '/SuperClassic') {
                Write-Host 'Installing Firebird SuperClassic...'
                $serverTypeComponent = 'ServerComponent\ClassicServerComponent,'
                $serverTypeTask = 'SuperClassicTask,'
            } elseif ($env:chocolateyPackageParameters -contains '/Classic') {
                Write-Host 'Installing Firebird Classic...'
                $serverTypeComponent = 'ServerComponent\ClassicServerComponent,'
                $serverTypeTask = ''
            } else {
                Write-Host 'Installing Firebird SuperServer...'
                $serverTypeComponent = 'ServerComponent\SuperServerComponent,'
                $serverTypeTask = ''
            }
        } else {
            # Firebird >= 3.0        
            if ($env:chocolateyPackageParameters -contains '/SuperClassic') {
                Write-Host 'Installing Firebird SuperClassic...'
                $serverTypeComponent = 'ServerComponent,'
                $serverTypeTask = 'UseSuperClassicTask,'
            } elseif ($env:chocolateyPackageParameters -contains '/Classic') {
                Write-Host 'Installing Firebird Classic...'
                $serverTypeComponent = 'ServerComponent,'
                $serverTypeTask = 'UseClassicServerTask,'
            } else {
                Write-Host 'Installing Firebird SuperServer...'
                $serverTypeComponent = 'ServerComponent,'
                $serverTypeTask = 'UseSuperServerTask,'
            }
        }

        # Server installation: always with admin components.
        $serverTypeComponent += 'DevAdminComponent,'
    }

    # Install server with legacy client authentication (3.0+) and copy fbclient.dll & gds32.dll into System folder.
    $installerArgs = '/SP-', 
                     '/VERYSILENT', 
                     '/NORESTART',
                     '/NOICONS',
                     '/SUPPRESSMSGBOXES',
                     '/COMPONENTS="' + $serverTypeComponent + 'ClientComponent"',
                     '/TASKS="' + $serverTypeTask + 'UseServiceTask,AutoStartTask,|UseGuardianTask,|InstallCPLAppletTask,|MenuGroupTask,CopyFbClientToSysTask,CopyFbClientAsGds32Task,EnableLegacyClientAuth"'

    Install-ChocolateyInstallPackage -PackageName $packageName -FileType $installerType `
                                     -SilentArgs $installerArgs -File $installerFullName
}


Uninstall-Firebird

